"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const api_1 = require("@vtex/api");
const conf_1 = require("../conf");
const env = require("../env");
const timeout_1 = require("../timeout");
const user_agent_1 = require("../user-agent");
const billingClient_1 = require("./billingClient");
const DEFAULT_TIMEOUT = 15000;
const context = {
    account: conf_1.getAccount(),
    authToken: conf_1.getToken(),
    production: false,
    region: env.region(),
    route: {
        id: '',
        params: {},
    },
    userAgent: user_agent_1.default,
    workspace: conf_1.getWorkspace() || 'master',
    requestId: '',
    operationId: '',
};
const options = {
    timeout: (timeout_1.default || DEFAULT_TIMEOUT),
};
const interceptor = (client) => new Proxy({}, {
    get: (_, name) => () => {
        throw new Error(`Error trying to call ${client}.${name.toString()} before login.`);
    },
});
const createClients = (customContext = {}, customOptions = {}) => {
    const mergedContext = Object.assign({}, context, customContext);
    const mergedOptions = Object.assign({}, options, customOptions);
    return {
        builder: new api_1.Builder(mergedContext, mergedOptions),
        logger: new api_1.Logger(mergedContext, mergedOptions),
        registry: new api_1.Registry(mergedContext, mergedOptions),
        events: new api_1.Events(mergedContext, mergedOptions),
    };
};
exports.createClients = createClients;
const [apps, router, workspaces, logger, events, billing] = conf_1.getToken()
    ? [
        new api_1.Apps(context, options),
        new api_1.Router(context, options),
        new api_1.Workspaces(context, options),
        new api_1.Logger(context),
        new api_1.Events(context),
        new billingClient_1.default(context, options),
    ]
    : [
        interceptor('apps'),
        interceptor('router'),
        interceptor('workspaces'),
        interceptor('logger'),
        interceptor('events'),
        interceptor('billing'),
    ];
exports.apps = apps;
exports.router = router;
exports.workspaces = workspaces;
exports.logger = logger;
exports.events = events;
exports.billing = billing;
