"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ramda_1 = require("ramda");
const clients_1 = require("../../../clients");
const utils_1 = require("../utils");
const getAppSettings = Promise.method(clients_1.apps.getAppSettings);
const saveAppSettings = Promise.method(clients_1.apps.saveAppSettings);
const castValue = value => {
    let parsedValue;
    try {
        parsedValue = JSON.parse(value);
    }
    catch (err) {
        parsedValue = value;
    }
    const numberCast = Number(value);
    return isNaN(numberCast) ? parsedValue : numberCast;
};
const transformCommandsToObj = (commandSettings) => {
    const k = [];
    const v = [];
    for (const [idx, val] of commandSettings.entries()) {
        const realValue = castValue(val);
        if (idx % 2) {
            v.push(realValue);
        }
        else {
            k.push(realValue);
        }
    }
    return ramda_1.zipObj(k, v);
};
exports.default = (app, _, ___, options) => {
    const commandSettings = transformCommandsToObj(utils_1.parseArgs(options._));
    return getAppSettings(app)
        .then(ramda_1.merge(ramda_1.__, commandSettings))
        .then(newSettings => JSON.stringify(newSettings, null, 2))
        .tap(newSettings => saveAppSettings(app, newSettings))
        .tap(console.log);
};
