"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const semver = require("semver");
const table_1 = require("../../table");
const clients_1 = require("../../clients");
const conf_1 = require("../../conf");
const logger_1 = require("../../logger");
const utils_1 = require("./utils");
const [account, workspace] = [conf_1.getAccount(), conf_1.getWorkspace()];
const { listAvailableServices, listInstalledServices, getAvailableVersions } = clients_1.router;
const printAvailableServices = () => listAvailableServices()
    .then((availableRes) => {
    const table = table_1.createTable({ head: ['Name', 'Last stable', 'Last prerelease'] });
    Object.keys(availableRes).forEach(res => {
        const [stable, prerelease] = utils_1.getLastStableAndPrerelease(availableRes[res]);
        table.push([res, chalk_1.default.bold.green(stable), chalk_1.default.yellow(prerelease)]);
    });
    return table;
})
    .then(table => {
    logger_1.default.info('Available services');
    console.log(table.toString());
});
const printAvailableServiceVersions = (name, filter) => getAvailableVersions(name)
    .then(({ versions }) => {
    const region = Object.keys(versions)[0];
    return versions[region]
        .filter(v => !filter || v.indexOf(filter) >= 0)
        .map(semver.valid)
        .filter(v => v !== null)
        .sort(semver.compare)
        .reverse()
        .slice(0, 20)
        .forEach(v => {
        if (semver.prerelease(v) !== null) {
            console.log(`  ${chalk_1.default.yellow(v)}`);
        }
        else {
            console.log(`  ${chalk_1.default.bold.green(v)}`);
        }
    });
});
const printInstalledServices = () => listInstalledServices()
    .then((installedRes) => {
    const table = table_1.createTable();
    installedRes.forEach(({ name, version }) => {
        const validVersion = semver.valid(version);
        const styledVersion = semver.prerelease(validVersion) !== null
            ? chalk_1.default.yellow(validVersion)
            : chalk_1.default.bold.green(validVersion);
        table.push([name, styledVersion]);
    });
    return table;
})
    .then(table => {
    logger_1.default.info(`Services installed on ${chalk_1.default.blue(account)}/${chalk_1.default.green(workspace)}`);
    console.log(table.toString());
});
exports.default = (name, options) => {
    const filter = options.f || options.filter;
    const available = options.a || options.available;
    return available
        ? (name ? printAvailableServiceVersions(name, filter) : printAvailableServices())
        : printInstalledServices();
};
