"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const moment = require("moment");
const numbro = require("numbro");
const R = require("ramda");
const conf_1 = require("../../../conf");
const logger_1 = require("../../../logger");
const table_1 = require("../../../table");
const utils_1 = require("./utils");
const formatPercent = (n) => numbro(n).format('0.000%');
const formatInteger = (n) => numbro(n).format('0,0');
const bold = (stringList) => R.map(chalk_1.default.bold)(stringList);
const printResultsTable = (testInfo) => {
    const { ABTestBeginning, WorkspaceA, WorkspaceB, Winner, WorkspaceASessions, WorkspaceBSessions, WorkspaceASessionsLast24Hours, WorkspaceBSessionsLast24Hours, ExpectedLossChoosingA, ExpectedLossChoosingB, ConversionA, ConversionALast24Hours, ConversionB, ConversionBLast24Hours, ProbabilityAlternativeBeatMaster, PValue, } = testInfo;
    console.log(chalk_1.default.bold(`VTEX AB Test: ${chalk_1.default.blue(`${WorkspaceA} (A)`)} vs ${chalk_1.default.blue(`${WorkspaceB} (B)`)}\n`));
    if (R.any(R.isNil)([ExpectedLossChoosingA, ExpectedLossChoosingB, ProbabilityAlternativeBeatMaster])) {
        logger_1.default.error('Unexpected value of conversion. Perhaps your user traffic is too small and this creates distortions in the data');
    }
    const rawDataTable = table_1.createTable();
    rawDataTable.push(bold(['', chalk_1.default.blue(WorkspaceA), chalk_1.default.blue(WorkspaceB)]));
    rawDataTable.push(bold(['Conversion', formatPercent(ConversionA), formatPercent(ConversionB)]));
    rawDataTable.push(bold(['Conversion (last 24h)',
        formatPercent(ConversionALast24Hours), formatPercent(ConversionBLast24Hours)]));
    rawDataTable.push(bold(['N. of Sessions', formatInteger(WorkspaceASessions), formatInteger(WorkspaceBSessions)]));
    rawDataTable.push(bold(['N. of Sessions (last 24h)',
        formatInteger(WorkspaceASessionsLast24Hours), formatInteger(WorkspaceBSessionsLast24Hours)]));
    const comparisonTable = table_1.createTable();
    comparisonTable.push(bold(['', chalk_1.default.blue(WorkspaceA), chalk_1.default.blue(WorkspaceB)]));
    comparisonTable.push(bold(['Expected Loss', formatPercent(ExpectedLossChoosingA), formatPercent(ExpectedLossChoosingB)]));
    const probabilitiesTable = table_1.createTable();
    probabilitiesTable.push(bold(['Event', 'Condition', 'Probability']));
    probabilitiesTable.push(bold(['B beats A', 'None', formatPercent(ProbabilityAlternativeBeatMaster)]));
    probabilitiesTable.push(bold(['Data as extreme as the observed', `Workspaces being equal (both to ${chalk_1.default.blue(WorkspaceA)}).`, formatPercent(PValue)]));
    const resultsTable = table_1.createTable();
    resultsTable.push(bold([`Start Date`, `${moment(ABTestBeginning).format('DD-MMM-YYYY HH:mm')} (UTC)`]));
    const nowUTC = moment.utc();
    const runningTime = nowUTC.diff(moment.utc(ABTestBeginning), 'minutes');
    resultsTable.push(bold([`Running Time`, utils_1.formatDuration(runningTime)]));
    resultsTable.push(bold([chalk_1.default.bold.green(`Winner`), chalk_1.default.bold.green(Winner)]));
    console.log(`Raw Data:\n${rawDataTable.toString()}\n`);
    console.log(`Comparison of losses in case of choosing wrong workspace:\n${comparisonTable.toString()}\n`);
    console.log(`Probabilities:\n${probabilitiesTable.toString()}\n`);
    console.log(`Results:\n${resultsTable.toString()}\n`);
};
exports.default = () => __awaiter(this, void 0, void 0, function* () {
    yield utils_1.installedABTester();
    let abTestInfo = [];
    abTestInfo = yield utils_1.abtester.status();
    if (!abTestInfo || abTestInfo.length === 0) {
        return logger_1.default.info(`No AB Tests running in account ${chalk_1.default.blue(conf_1.getAccount())}\n`);
    }
    R.map(printResultsTable, abTestInfo);
});
