"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const ramda_1 = require("ramda");
const clients_1 = require("../../clients");
const conf_1 = require("../../conf");
const errors_1 = require("../../errors");
const logger_1 = require("../../logger");
const utils_1 = require("../apps/utils");
const prompts_1 = require("../prompts");
const use_1 = require("./use");
const account = conf_1.getAccount();
const workspace = conf_1.getWorkspace();
const promptWorkspaceDeletion = (names) => prompts_1.promptConfirm(`Are you sure you want to delete workspace` + (names.length > 1 ? 's' : '') + ` ${chalk_1.default.green(names.join(', '))}?`, true);
exports.deleteWorkspaces = (names = []) => __awaiter(this, void 0, void 0, function* () {
    const name = ramda_1.head(names);
    const decNames = ramda_1.tail(names);
    if (names.length === 0) {
        return [];
    }
    logger_1.default.debug('Starting to delete workspace', name);
    try {
        yield clients_1.workspaces.delete(account, name);
        logger_1.default.info(`Workspace ${chalk_1.default.green(name)} deleted ${chalk_1.default.green('successfully')}`);
        return ramda_1.flatten([name, yield exports.deleteWorkspaces(decNames)]);
    }
    catch (err) {
        logger_1.default.warn(`Workspace ${chalk_1.default.green(name)} was ${chalk_1.default.red('not')} deleted`);
        logger_1.default.error(`Error ${err.response.status}: ${err.response.statusText}. ${err.response.data.message}`);
        return exports.deleteWorkspaces(decNames);
    }
});
exports.default = (name, options) => __awaiter(this, void 0, void 0, function* () {
    const names = ramda_1.prepend(name, utils_1.parseArgs(options._));
    const preConfirm = options.y || options.yes;
    const force = options.f || options.force;
    logger_1.default.debug('Deleting workspace' + (names.length > 1 ? 's' : '') + ':', names.join(', '));
    if (!force && ramda_1.contains(workspace, names)) {
        return logger_1.default.error(`You are currently using the workspace ${chalk_1.default.green(workspace)}, please change your workspace before deleting`);
    }
    if (!preConfirm && !(yield promptWorkspaceDeletion(names))) {
        throw new errors_1.UserCancelledError();
    }
    const deleted = yield exports.deleteWorkspaces(names);
    if (ramda_1.contains(workspace, deleted)) {
        logger_1.default.warn(`The workspace you were using was deleted`);
        return yield use_1.default('master');
    }
});
